/* ---------------------------------------------------------- */
/*  tcgs_cave_3d.cpp                                          */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief    3D cave race - game main
    @author   rerofumi
    @since    Nov.08.2006
*/
/*-----------------------------------------------------
 Copyright (C) 2006 fumi2kick/rerofumi <rero2@yuumu.org>
 All Rights Reserved.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/
#include <stdio.h>
#include "tcgs_cave_3d.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

#define MODEL_INTERVAL 1.5

//! vC[̑傫Aa
#define MYSHIP_SIZE  0.2

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

GLfloat glfLightAmbient[]  = {0.1, 0.1, 0.1, 1.0};
GLfloat glfLightDiffuse[]  = {0.9, 0.9, 0.9, 1.0};
GLfloat glfLightSpecular[] = {0.0, 0.0, 0.0, 1.0};
GLfloat glfLightPosition[] = {-0.3, 0.7, 1.0, 0.0};

GLfloat glfLightDiffuse_red[]  = {0.9, 0.0, 0.0, 1.0};

GLfloat stage_color[ALLSTAGE][4] = {
  {1.0, 0.8, 0.0, 1.0},
  {0.7, 0.3, 0.2, 1.0},
  {0.5, 0.6, 1.0, 1.0},
};

char *stage_data_file[3] = {
  "stage_01.csv",
  "stage_02.csv",
  "stage_03.csv",
};

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* constructor/destructor                               */
/*------------------------------------------------------*/

/**
 * RXgN^
 */
Cave3dGame::Cave3dGame()
{
}

Cave3dGame::Cave3dGame(PakoOpenglApp *parent)
{
  int  i;

  m_parent = parent;

  /* --- R[h */
  for(int i=0; i<ALLSTAGE; i++) {
    m_record[i] = 100*60*60 - 1;
  }
  /* --- R[Xf */
  for(i=0; i<MODEL_NUM; i++) {
    model[i] = new CaveModel;
    model[i]->x = 0.0;
    model[i]->y = 0.0;
    model[i]->size = 0.0;
    model[i]->hole_w = -1.0;
    model[i]->hole_h = -1.5;
  }
  stage_data.clear();
}

/**
 * fXgN^(n)
 */
Cave3dGame::~Cave3dGame()
{
  for(int i=0; i<MODEL_NUM; i++) {
    delete model[i];
  }
}


/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

void Cave3dGame::setParent(PakoOpenglApp *parent)
{
  m_parent = parent;
}

int Cave3dGame::initalizeGame(void)
{
  PakoSpriteItem *obj;
  int i;

  m_clock = 0;
  m_nowPlaying = 0;
  m_gameStep = 0;
  m_gameStage = 1;

  m_playerPosition = -1.0;

  /* --- XvCg */
  for(i=0; i<64; i++) {
    obj = m_parent->getSpriteItem(1, i);
    obj->m_Visible = false;
  }

  for(i=0; i<ALLSTAGE; i++) {
    m_result[i] = 0;
  }

  SoundMusicStop();

  /* --- R[h^Cǂݍ */
  load_record_time();

  return(0);
}

int Cave3dGame::initializeStage(int stage)
{
  PakoSpriteItem *obj;
  FILE *fp;
  CaveModel in;
  int  i;

  stage_data.clear();
  m_forword = 0;
  m_playerShield = 1.0;
  m_stageTime = 0;
  m_beforeStart = 300;

  m_playerX = m_playerY = 0;
  m_playerRoll = 0.0;
  m_playerMoveX = m_playerMoveY = 0;
  m_playerMoveRoll = 0.0;
  m_playerSpeed = 0.5;
  m_playerForword = 0.0;

  for(i=0; i<MODEL_NUM; i++) {
    model[i]->x = 0.0;
    model[i]->y = 0.0;
    model[i]->size = 0.0;
    model[i]->hole_w = -1.0;
    model[i]->hole_h = -1.5;
  }

  /* --- XvCg */
  for(i=0; i<64; i++) {
    obj = m_parent->getSpriteItem(1, i);
    obj->m_Visible = false;
  }

  /* --- BGM () */
  m_stageMusic = 3 + (m_gameStage - 1);

  /* ---Xe[Wf[^ǂݍ() */
  fp = fopen(stage_data_file[m_gameStage-1], "r");
  if (fp != 0) {
    while(fscanf(fp, "%f,%f,%f,%f,%f", &in.x, &in.y, &in.size, &in.hole_w, &in.hole_h) == 5) {
      stage_data.push_back(in);
    }
  }
  fclose(fp);

  m_stageColor[0] = stage_color[m_gameStage-1][0];
  m_stageColor[1] = stage_color[m_gameStage-1][1];
  m_stageColor[2] = stage_color[m_gameStage-1][2];
  m_stageColor[3] = stage_color[m_gameStage-1][3];

  return(0);
}

int Cave3dGame::getState(void)
{
  return(m_nowPlaying);
}



/* ----------------------------------------- */
/* --- tick() */
/**
 * t[ĂяoQ[{̃\bh
 * pŃI[o[ChĂɃQ[{̂Lq
 * 1t[̍ƂI烁\bh𔲂邱
 *
 * @return trueŒʏƒAfalseŃQ[I
 *
 */
bool Cave3dGame::tick(void)
{
  int  key;

  m_clock += 1;
  m_beforeStart -= 1;


  switch(m_gameStep) {
    /* --- Q[ */
  case 0:
    SoundMusicStop();
    initializeStage(m_gameStage);
    m_gameStep = 1;

    /* --- get ready? */
  case 1:
    if (m_beforeStart == 240) {
      SoundSE(2);
    }
    if (m_beforeStart == 0) {
      SoundSE(3);
      SoundMusic(m_stageMusic);
      m_gameStep = 2;
    }
    m_playerForword += (m_playerSpeed * 0.1);
    if (m_playerForword > 1.0) {
      m_playerForword -= 1.0;
      forword_step();
    }
    break;

    /* --- game main*/
  case 2:
    m_stageTime += 1;
    myship_move();
    m_playerForword += (m_playerSpeed * 0.1);
    if (m_playerForword > 1.0) {
      m_playerForword -= 1.0;
      if (forword_step() != 0) {
	/* --- clash retire */
	SoundMusicStop();
	/* se: crash gameover */
	SoundSE(6);
	m_gameoverTimer = 0;
	m_gameStep = 4;
      }
    }
    if ((m_forword-MODEL_NUM) == (int)stage_data.size()) {
      m_result[m_gameStage-1] = m_stageTime;
       /* record check */
      if (m_record[m_gameStage-1] >= m_stageTime) {
	m_record[m_gameStage-1] = m_stageTime;
	save_record_time();
      }
      /* se: clear */
      SoundSE(4);
      m_afterClear = 0;
      m_gameStep = 3;
    }
    break;

    /* --- clear */
  case 3:
    m_afterClear += 1;
    sprite_goal_disp(m_afterClear);
    if (m_afterClear == 240) {
      m_gameStage += 1;
      if (m_gameStage == (ALLSTAGE+1)) {
	SoundMusicStop();
	/* se: all clear */
	SoundSE(5);
	m_gameoverTimer = 0;
	m_gameStep = 5;
      }
      else {
	m_gameStep = 0;
      }
    }
    break;

    /* --- game over */
  case 4:
    sprite_retire_disp();
    m_gameoverTimer += 1;
    if (m_gameoverTimer > 240) {
      m_nowPlaying = 1;
    }
    break;

    /* --- all clear */
  case 5:
    key = InputJoyKeyTriger(0);
    m_gameoverTimer += 1;
    if ((m_gameoverTimer > 60) &&
      ((key & (IN_Button1|IN_Button2)) != 0)) {
      m_nowPlaying = 1;
    }
    break;

  }

  /* -- eXvCg\ */
  if (m_gameStep == 5) {
    sprite_allclear_disp(m_gameoverTimer);
  }
  else {
    sprite_disp();
  }

  return true;
}


/* ----------------------------------------- */
/* --- tickDisplay() */
/**
 * t[Ăяoʕ\\bh
 * pŃI[o[ChĂɉʕ`揈Lq
 * tick()̒ɋLqĂǂA̓t[XLbv
 * ΂Ă΂Ȃ̂ŏ肭p
 *
 * @return trueŒʏƒAfalseŕ\ُ
 *
 */
bool Cave3dGame::tickDisplay(void)
{
  int  i;
  float  myship_forword;
  GLfloat col[4], backcol;

  /* --- KȌ */
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  if (m_gameStep != 4) {
    //glLightfv(GL_LIGHT0, GL_AMBIENT,  glfLightAmbient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  glfLightDiffuse);
    //glLightfv(GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
    glLightfv(GL_LIGHT0, GL_POSITION, glfLightPosition);
  }
  else {
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  glfLightDiffuse_red);
    glLightfv(GL_LIGHT0, GL_POSITION, glfLightPosition);
  }

  /* -- vC[Ԃ烏[h̐ݒ */
  myship_forword = m_playerForword * MODEL_INTERVAL;
  glRotatef(-m_playerRoll, 0, 0, 2.0);
  glTranslatef(m_playerX, m_playerY, myship_forword);
  
   /* -- ]Aj[Vݒ */
  for(i=0; i<MODEL_NUM; i++) {
    col[0] = m_stageColor[0] * (MODEL_NUM - i) / MODEL_NUM;
    col[1] = m_stageColor[1] * (MODEL_NUM - i) / MODEL_NUM;
    col[2] = m_stageColor[2] * (MODEL_NUM - i) / MODEL_NUM;
    col[3] = 1.0;
    glMaterialfv(GL_FRONT, GL_DIFFUSE, col);
    gl_disp_frame(model[i]->size, model[i]->hole_w, model[i]->hole_h);
    glTranslatef(model[i]->x, model[i]->y, -MODEL_INTERVAL);
  }
  glDisable(GL_LIGHTING);

  /* -- NA̔wi */
  if (m_forword > (int)stage_data.size()) {
    backcol = (float)(m_forword - stage_data.size()) / (float)MODEL_NUM;
    if (backcol > 0.8) {
      backcol = 0.8;
    }
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    m_parent->selectTexture(0);
    glColor4f(backcol, backcol, backcol, 1.0);
    glBegin(GL_POLYGON);
    glTexCoord2f(0, 328.0/512.0);
    glVertex3f(-50.0, 50.0, -100.0);
    glTexCoord2f(0, (328.0+136.0)/512.0);
    glVertex3f(-50.0,-50.0, -100.0);
    glTexCoord2f((240.0/512.0), (328.0+136.0)/512.0);
    glVertex3f( 50.0,-50.0, -100.0);
    glTexCoord2f((240.0/512.0), (328.0/512.0));
    glVertex3f(50.0, 50.0, -100.0);
    glEnd();
  }

  return true;
}


/*------------------------------------------------------*/
/* private method                                       */
/*------------------------------------------------------*/

/* ----------------------------------------- */
/* --- gl_disp_frame() */
/**
 * Ǎf\
 *
 * @param [in] size Ŝ̑傫
 * @param [in] hole ɋ󂢂Ă錊̑傫
 *
 */
void Cave3dGame::gl_disp_frame(float size, float hole_w, float hole_h)
{
  float hx1, hy1, hx2, hy2;
  float sx1, sy1, sx2, sy2;
  GLdouble norm[3];

  if ((size < 0) || (hole_w < 0) || (hole_h < 0)) {
    return;
  }

  hx1 = -(hole_w * 0.5);
  hy1 =  (hole_h * 0.5);
  hx2 =  (hole_w * 0.5);
  hy2 = -(hole_h * 0.5);
  sx1 = -(size * 0.5);
  sy1 =  (size * 0.5);
  sx2 = sy1;
  sy2 = sx1;

  /* ---  to  */
  glBegin(GL_POLYGON);
  norm[0]=1; norm[1]=0; norm[2]=0;
  glNormal3dv (norm);
  glVertex3f(hx1, hy1, -1.0);
  glVertex3f(hx1, hy1, 0.0);
  glVertex3f(hx1, hy2, 0.0);
  glVertex3f(hx1, hy2, -1.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=1; norm[2]=0;
  glNormal3dv (norm);
  glVertex3f(hx1, hy2, -1.0);
  glVertex3f(hx1, hy2, 0.0);
  glVertex3f(hx2, hy2, 0.0);
  glVertex3f(hx2, hy2, -1.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=-1; norm[1]=0; norm[2]=0;
  glNormal3dv (norm);
  glVertex3f(hx2, hy2, -1.0);
  glVertex3f(hx2, hy2, 0.0);
  glVertex3f(hx2, hy1, 0.0);
  glVertex3f(hx2, hy1, -1.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=-1; norm[2]=0;
  glNormal3dv (norm);
  glVertex3f(hx2, hy1, -1.0);
  glVertex3f(hx2, hy1, 0.0);
  glVertex3f(hx1, hy1, 0.0);
  glVertex3f(hx1, hy1, -1.0);
  glEnd();
  /* ---  */
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=0; norm[2]=1;
  glNormal3dv (norm);
  glVertex3f(sx1, sy1, 0.0);
  glVertex3f(sx1, sy2, 0.0);
  glVertex3f(hx1, hy2, 0.0);
  glVertex3f(hx1, hy1, 0.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=0; norm[2]=1;
  glNormal3dv (norm);
  glVertex3f(hx1, hy2, 0.0);
  glVertex3f(sx1, sy2, 0.0);
  glVertex3f(sx2, sy2, 0.0);
  glVertex3f(hx2, hy2, 0.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=0; norm[2]=1;
  glNormal3dv (norm);
  glVertex3f(hx2, hy2, 0.0);
  glVertex3f(sx2, sy2, 0.0);
  glVertex3f(sx2, sy1, 0.0);
  glVertex3f(hx2, hy1, 0.0);
  glEnd();
  glBegin(GL_POLYGON);
  norm[0]=0; norm[1]=0; norm[2]=1;
  glNormal3dv (norm);
  glVertex3f(hx2, hy1, 0.0);
  glVertex3f(sx2, sy1, 0.0);
  glVertex3f(sx1, sy1, 0.0);
  glVertex3f(hx1, hy1, 0.0);
  glEnd();

}


/* ----------------------------------------- */
/* --- forword_step() */
/**
 * Xe[W̃ff[^ 1Xebvi߂
 *
 */
int Cave3dGame::forword_step(void)
{
  CaveModel *tmp;
  float  near, damage;
  int  clash;

  clash = 0;

  /* --- vC[Έʒu␳ */
  m_playerX += model[0]->x;
  m_playerY += model[0]->y;

  /* --- fVtg */
  tmp = model[0];
  for(int i=0; i<(MODEL_NUM-1); i++) {
    model[i] = model[i+1];
  }
  model[MODEL_NUM-1] = tmp;
  model[MODEL_NUM-1]->x = 0.0;
  model[MODEL_NUM-1]->y = 0.0;
  model[MODEL_NUM-1]->size = -1;
  model[MODEL_NUM-1]->hole_w = -1;
  model[MODEL_NUM-1]->hole_h = -1;

  /* --- f[^i߂ */
  if (m_forword < (int)stage_data.size()) {
    model[MODEL_NUM-1]->x = stage_data[m_forword].x;
    model[MODEL_NUM-1]->y = stage_data[m_forword].y;
    model[MODEL_NUM-1]->size = stage_data[m_forword].size;
    model[MODEL_NUM-1]->hole_w = stage_data[m_forword].hole_w;
    model[MODEL_NUM-1]->hole_h = stage_data[m_forword].hole_h;
  }

  /* --- 蔻 */
  if ((model[0]->hole_w > 0) && (model[0]->hole_h > 0)) {
    damage = 0;
    near = ((model[0]->hole_w * 0.5) - MYSHIP_SIZE);
    if (m_playerX > near) {
      damage = (m_playerX - near) / MYSHIP_SIZE;
      m_playerMoveX += -0.040 * damage;
    }
    if (m_playerX < -near) {
      damage = -(m_playerX + near) / MYSHIP_SIZE;
      m_playerMoveX += 0.040 * damage;
    }
    near = ((model[0]->hole_h * 0.5) - MYSHIP_SIZE);
    if (m_playerY > near) {
      damage = (m_playerY - near) / MYSHIP_SIZE;
      m_playerMoveY += -0.040 * damage;
    }
    if (m_playerY < -near) {
      damage = -(m_playerY + near) / MYSHIP_SIZE;
      m_playerMoveY += 0.040 * damage;
    }
    if (damage > 0) {
      if (m_playerShield == 0.0) {
	clash = 1;
      }
      else {
	SoundSE(1);
	m_playerSpeed -= (0.8 * damage);
	if (m_playerSpeed < 0.2) {
	  m_playerSpeed = 0.2;
	}
	m_playerShield -= (0.5 * damage);
	if (m_playerShield < 0.0) {
	  m_playerShield = 0.0;
	}
      }
    }
  }

  m_forword += 1;
  return(clash);
}


/* ----------------------------------------- */
/* --- myship_move() */
/**
 * vC[̈ړ
 *
 */
void Cave3dGame::myship_move(void)
{
  float  roll;
  float  add_x, add_y;
  int  key;

  roll = m_playerRoll * 65535.0 / 360.0;
  add_x = add_y = 0;
  key = InputJoyKey(0);
  /* --- ㉺E */
  if ((key & IN_Up) != 0) {
    add_x += 0.0;
    add_y += 0.0015;
  }
  if ((key & IN_Down) != 0) {
    add_x += 0.0;
    add_y += -0.0015;
  }
  if ((key & IN_Left) != 0) {
    add_x += 0.0010;
    add_y += 0.0;
  }
  if ((key & IN_Right) != 0) {
    add_x += -0.0010;
    add_y += 0.0;
  }
  m_playerMoveX += add_x;
  m_playerMoveY += add_y;
  /* --- x~b^[ */
  if (m_playerMoveX > 0.008) {
    m_playerMoveX = 0.008;
  }
  if (m_playerMoveX < -0.008) {
    m_playerMoveX = -0.008;
  }
  if (m_playerMoveY > 0.01) {
    m_playerMoveY = 0.01;
  }
  if (m_playerMoveY < -0.013) {
    m_playerMoveY = -0.013;
  }
  /* --- R~ */
  if (m_playerMoveX > 0) {
    if (m_playerMoveX > 0.0005) {
      m_playerMoveX -= 0.0005;
    }
    else {
      m_playerMoveX = 0.0;
    }
  }
  if (m_playerMoveX < 0) {
    if (m_playerMoveX < -0.0005) {
      m_playerMoveX += 0.0005;
    }
    else {
      m_playerMoveX = 0.0;
    }
  }
  if (m_playerMoveY > 0) {
    if (m_playerMoveY > 0.0005) {
      m_playerMoveY -= 0.0005;
    }
    else {
      m_playerMoveY = 0.0;
    }
  }
  if (m_playerMoveY < 0) {
    if (m_playerMoveY < -0.0005) {
      m_playerMoveY += 0.0005;
    }
    else {
      m_playerMoveY = 0.0;
    }
  }
  /* --- ړxZ */
  m_playerX +=
    (m_playerMoveX * PakoMath::Cos((long)roll)) - 
    (m_playerMoveY * PakoMath::Sin((long)roll));
  m_playerY +=
    (m_playerMoveX * PakoMath::Sin((long)roll)) +
    (m_playerMoveY * PakoMath::Cos((long)roll));
  /* --- ʕǔf */
  if ((model[0]->hole_w > 0) && (model[0]->hole_h > 0)) {
    if (m_playerX > (model[0]->hole_w * 0.5)) {
      m_playerX = (model[0]->hole_w * 0.5);
    }
    if (m_playerX < -(model[0]->hole_w * 0.5)) {
      m_playerX = -(model[0]->hole_w * 0.5);
    }
    if (m_playerY > (model[0]->hole_h * 0.5)) {
      m_playerY = (model[0]->hole_h * 0.5);
    }
    if (m_playerY < -(model[0]->hole_h * 0.5)) {
      m_playerY = -(model[0]->hole_h * 0.5);
    }
  }

  /* --- [{^ */
  if ((key & IN_Button5) != 0) {
    m_playerMoveRoll += 0.15;
  }
  if ((key & IN_Button6) != 0) {
    m_playerMoveRoll -= 0.15;
  }
  /* -- x~b^[ */
  if (m_playerMoveRoll > 1.250) {
    m_playerMoveRoll = 1.250;
  }
  if (m_playerMoveRoll < -1.250) {
    m_playerMoveRoll = -1.250;
  }
  /* --- R~ */
  if (m_playerMoveRoll > 0) {
    if (m_playerMoveRoll > 0.04) {
      m_playerMoveRoll -= 0.04;
    }
    else {
      m_playerMoveRoll = 0.0;
    }
  }
  if (m_playerMoveRoll < 0) {
    if (m_playerMoveRoll < -0.04) {
      m_playerMoveRoll += 0.04;
    }
    else {
      m_playerMoveRoll = 0.0;
    }
  }
  /* -- [xZ */
  m_playerRoll += m_playerMoveRoll;
  if (m_playerRoll > 360.0) {
    m_playerRoll -= 360.0;
  }
  if (m_playerRoll < 0.0) {
    m_playerRoll += 360.0;
  }

  /* --- ANZ */
  if ((key & (IN_Button1 | IN_Button2)) != 0) {
    if (m_playerSpeed < 0.6) {
      m_playerSpeed += 0.005;
    }
    else if (m_playerSpeed < 0.8) {
      m_playerSpeed += 0.002;
    }
    else if (m_playerSpeed < 0.9) {
      m_playerSpeed += 0.001;
    }
    else {
      m_playerSpeed += 0.0003;
    }
    /* - ~b^[ */
    if (m_playerSpeed > 1.0) {
      m_playerSpeed = 1.0;
    }
  }
  else {
    /* - mANZ̃u[L */
    m_playerSpeed -= 0.002;
    /* - ~b^[ */
    if (m_playerSpeed < 0.1) {
      m_playerSpeed = 0.1;
    }
  }
  /* --- u[L */
  if ((key & (IN_Button3 | IN_Button4)) != 0) {
    m_playerSpeed -= 0.01;
    /* - ~b^[ */
    if (m_playerSpeed < 0.1) {
      m_playerSpeed = 0.1;
    }
  }

}



/* ----------------------------------------- */
/* --- sprite_disp() */
/**
 * eXvCgp[c\
 *
 */
void Cave3dGame::sprite_disp(void)
{
  PakoSpriteItem *obj;
  int  speed, damage;

  /* --- Xs[h[^[ */
  obj = m_parent->getSpriteItem(1, 0);
  obj->m_PosX = 360;
  obj->m_PosY = 32;
  obj->m_SizeW = 120;
  obj->m_SizeH = 240;
  obj->m_Blend = 0.4;
  obj->m_TexU = 391;
  obj->m_TexV = 271;
  obj->m_TexW = 120;
  obj->m_TexH = 240;
  obj->m_RollZ = 0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* */
  speed = (int)(24.0 * m_playerSpeed);
  obj = m_parent->getSpriteItem(1, 1);
  obj->m_PosX = 360+73;
  obj->m_PosY = 32+12 + (24 - speed) * 8;
  obj->m_SizeW = 40;
  obj->m_SizeH = speed * 8;
  obj->m_Blend = 0.8;
  obj->m_TexU = 472;
  obj->m_TexV = (24 - speed) * 8;
  obj->m_TexW = 40;
  obj->m_TexH = speed * 8;
  obj->m_RollZ = 0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;

  /* --- HUD */
  obj = m_parent->getSpriteItem(1, 2);
  obj->m_PosX = 240;
  obj->m_PosY = 136;
  obj->m_SizeW = 128;
  obj->m_SizeH = 128;
  obj->m_Blend = 0.4;
  obj->m_TexU = 320;
  obj->m_TexV = 24;
  obj->m_TexW = 128;
  obj->m_TexH = 128;
  obj->m_RollZ = m_playerRoll / 360.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = true;
  obj->m_Visible = true;
  obj->m_TextureID = 0;

  /* --- Shield */
  obj = m_parent->getSpriteItem(1, 3);
  obj->m_PosX = 0 + 8;
  obj->m_PosY = 232 - 8;
  obj->m_SizeW = 64;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.4;
  obj->m_TexU = 192;
  obj->m_TexV = 24;
  obj->m_TexW = 64;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  damage = (int)(128.0 * m_playerShield);
  obj = m_parent->getSpriteItem(1, 4);
  obj->m_PosX = 0 + 8;
  obj->m_PosY = 256 - 8;
  obj->m_SizeW = damage;
  obj->m_SizeH = 16;
  obj->m_Blend = 0.8;
  obj->m_TexU = 192;
  obj->m_TexV = 64;
  obj->m_TexW = damage;
  obj->m_TexH = 16;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  obj = m_parent->getSpriteItem(1, 5);
  obj->m_PosX = damage + 8;
  obj->m_PosY = 256 - 8;
  obj->m_SizeW = 128-damage;
  obj->m_SizeH = 16;
  obj->m_Blend = 0.4;
  obj->m_TexU = 192+damage;
  obj->m_TexV = 48;
  obj->m_TexW = 128-damage;
  obj->m_TexH = 16;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;

  /* --- Start */
  obj = m_parent->getSpriteItem(1, 6);
  obj->m_PosX = 240;
  obj->m_PosY = 136;
  obj->m_SizeW = 168;
  obj->m_SizeH = 48;
  obj->m_Blend = 0.6;
  obj->m_TexU = 0;
  obj->m_TexV = 72;
  obj->m_TexW = 168;
  obj->m_TexH = 48;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = true;
  obj->m_TextureID = 0;
  if ((m_beforeStart <= 0) && (m_beforeStart > -120)) {
    obj->m_Visible = true;
  }
  else {
    obj->m_Visible = false;
  }

  /* --- Time */
  obj = m_parent->getSpriteItem(1, 7);
  obj->m_PosX = 8;
  obj->m_PosY = 8;
  obj->m_SizeW = 56;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.4;
  obj->m_TexU = 192;
  obj->m_TexV = 0;
  obj->m_TexW = 56;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  time_disp(64, 8, m_stageTime, 8);

  /* --- clear */
  if (m_gameStep == 3) {
    obj = m_parent->getSpriteItem(1, 2);
    obj->m_PosX = 240;
    obj->m_PosY = 80;
    obj->m_SizeW = 240;
    obj->m_SizeH = 56;
    obj->m_Blend = 0.6;
    obj->m_TexU = 0;
    obj->m_TexV = 120;
    obj->m_TexW = 240;
    obj->m_TexH = 56;
    obj->m_RollZ = 0.0;
    obj->m_Zoom = 1.0;
    obj->m_PosCenter = true;
    obj->m_Visible = true;
    obj->m_TextureID = 0;
  }
}


/* ----------------------------------------- */
/* --- time_disp() */
/**
 * bv^C\p
 *
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param frame [in] \ԃX^[g̃t[
 * @param sp [in] gpXvCg̐擪ԍ
 */
void Cave3dGame::time_disp(int x, int y, int frame, int sp)
{
  PakoSpriteItem *obj;
  int  keta, b, mx;
  
  mx = x;
  b = frame;
  /* --- min */
  keta = b / 36000;
  b = b % 36000;
  obj = m_parent->getSpriteItem(1, sp);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* */
  mx += 16;
  keta = b / 3600;
  b = b % 3600;
  obj = m_parent->getSpriteItem(1, sp+1);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* : */
  mx += 16;
  obj = m_parent->getSpriteItem(1, sp+2);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 408;
  obj->m_TexV = 0;
  obj->m_TexW = 8;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* sec */
  mx += 16;
  keta = b / 600;
  b = b % 600;
  obj = m_parent->getSpriteItem(1, sp+3);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* */
  mx += 16;
  keta = b / 60;
  b = b % 60;
  obj = m_parent->getSpriteItem(1, sp+4);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* . */
  mx += 16;
  obj = m_parent->getSpriteItem(1, sp+5);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 416;
  obj->m_TexV = 0;
  obj->m_TexW = 8;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* ms */
  b = 1000 * b / 60;
  mx += 16;
  keta = b / 100;
  b = b % 100;
  obj = m_parent->getSpriteItem(1, sp+6);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  mx += 16;
  keta = b / 10;
  b = b % 10;
  obj = m_parent->getSpriteItem(1, sp+7);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  mx += 16;
  keta = b;
  obj = m_parent->getSpriteItem(1, sp+8);
  obj->m_PosX = mx;
  obj->m_PosY = y;
  obj->m_SizeW = 16;
  obj->m_SizeH = 24;
  obj->m_Blend = 0.75;
  obj->m_TexU = 248 + (keta * 16);
  obj->m_TexV = 0;
  obj->m_TexW = 16;
  obj->m_TexH = 24;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;

}


/* ----------------------------------------- */
/* --- sprite_goal_disp() */
/**
 * NA̕\ACe
 *
 * @param timer [in] NÃ^C
 */
void Cave3dGame::sprite_goal_disp(int timer)
{
  PakoSpriteItem *obj;

  /* --- record display */
  if (timer > 60) {
    if (m_record[m_gameStage-1] == m_stageTime) {
      /* -- new record */
      obj = m_parent->getSpriteItem(1, 18);
      obj->m_PosX = 240;
      obj->m_PosY = 140;
      obj->m_SizeW = 112;
      obj->m_SizeH = 24;
      obj->m_Blend = 0.75;
      obj->m_TexU = 192;
      obj->m_TexV = 80;
      obj->m_TexW = 112;
      obj->m_TexH = 24;
      obj->m_RollZ = 0.0;
      obj->m_Zoom = 1.0;
      obj->m_PosCenter = true;
      obj->m_Visible = true;
      obj->m_TextureID = 0;
    }
    else {
      /* -- record display */
      obj = m_parent->getSpriteItem(1, 18);
      obj->m_PosX = 90;
      obj->m_PosY = 140;
      obj->m_SizeW = 74;
      obj->m_SizeH = 24;
      obj->m_Blend = 0.75;
      obj->m_TexU = 234;
      obj->m_TexV = 80;
      obj->m_TexW = 74;
      obj->m_TexH = 24;
      obj->m_RollZ = 0.0;
      obj->m_Zoom = 1.0;
      obj->m_PosCenter = false;
      obj->m_Visible = true;
      obj->m_TextureID = 0;
      time_disp(90+80, 140, m_record[m_gameStage-1], 19);
    }
  }
}


/* ----------------------------------------- */
/* --- sprite_retire_disp() */
/**
 * NbV^CA̕\ACe
 *
 */
void Cave3dGame::sprite_retire_disp(void)
{
  PakoSpriteItem *obj;

  obj = m_parent->getSpriteItem(1, 18);
  obj->m_PosX = 240;
  obj->m_PosY = 136;
  obj->m_SizeW = 120;
  obj->m_SizeH = 40;
  obj->m_Blend = 0.60;
  obj->m_TexU = 240;
  obj->m_TexV = 208;
  obj->m_TexW = 120;
  obj->m_TexH = 40;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = true;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
}


/* ----------------------------------------- */
/* --- sprite_goal_disp() */
/**
 * NA̕\ACe
 *
 * @param timer [in] \JE^[
 */
void Cave3dGame::sprite_allclear_disp(int timer)
{
  PakoSpriteItem *obj;
  int  i;

  /* --- XvCg */
  for(i=0; i<64; i++) {
    obj = m_parent->getSpriteItem(1, i);
    obj->m_Visible = false;
  }

  /* --- all clear */
  obj = m_parent->getSpriteItem(1, 0);
  obj->m_PosX = 60;
  obj->m_PosY = 32;
  obj->m_SizeW = 120;
  obj->m_SizeH = 56;
  obj->m_Blend = 1.0;
  obj->m_TexU = 240;
  obj->m_TexV = 152;
  obj->m_TexW = 120;
  obj->m_TexH = 56;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;
  /* */
  obj = m_parent->getSpriteItem(1, 1);
  obj->m_PosX = 180;
  obj->m_PosY = 32;
  obj->m_SizeW = 240;
  obj->m_SizeH = 56;
  obj->m_Blend = 1.0;
  obj->m_TexU = 0;
  obj->m_TexV = 120;
  obj->m_TexW = 240;
  obj->m_TexH = 56;
  obj->m_RollZ = 0.0;
  obj->m_Zoom = 1.0;
  obj->m_PosCenter = false;
  obj->m_Visible = true;
  obj->m_TextureID = 0;

  /* --- result */
  for(i=0; i<ALLSTAGE; i++) {
    obj = m_parent->getSpriteItem(1, 2+i*10);
    obj->m_PosX = 110;
    obj->m_PosY = 130 + i*40;
    obj->m_SizeW = 48;
    obj->m_SizeH = 24;
    obj->m_Blend = 1.0;
    obj->m_TexU = 368;
    obj->m_TexV = 160 + 24*i;
    obj->m_TexW = 48;
    obj->m_TexH = 24;
    obj->m_RollZ = 0.0;
    obj->m_Zoom = 1.0;
    obj->m_PosCenter = false;
    obj->m_Visible = true;
    obj->m_TextureID = 0;
    /* */
    time_disp(110+48+40, (128 + i*40), m_result[i], 3+i*10);
  }

}




/* ----------------------------------------- */
/* --- load_record_time() */
/**
 * Xe[WR[h^C̃t@Cǂݍ
 *
 */
void Cave3dGame::load_record_time(void)
{
  FILE *fp;
  char  line[128];
  int  rec;

  fp = fopen("record.txt", "r");
  if (fp != 0) {
    for(int i=0; i<3; i++) {
      if (fgets(line, 127, fp) == 0) break;
      sscanf(line, "%d", &rec);
      m_record[i] = rec;
    }
    fclose(fp);
  }
}


/* ----------------------------------------- */
/* --- save_record_time() */
/**
 * Xe[WR[h^C̃t@C
 *
 */
void Cave3dGame::save_record_time(void)
{
  FILE *fp;

  fp = fopen("record.txt", "w");
  if (fp != 0) {
    for(int i=0; i<3; i++) {
      fprintf(fp, "%d\n", m_record[i]);
    }
    fclose(fp);
  }
}

